/*
 * LCD430.c
 *
 *  Created on: 26/nov/2013
 *
 *		MSP430  	LCD_Module
 *		P2.4			RS
 *		P2.3			E
 *	La porta dati  connessa al PCF8574A
 *
 *		MSP430
 *		P1.7	SDA
 *		P1.6	SCL
 *
 */

#include <msp430.h>
#include "LCD430.h"


 extern char disp[16];  // Buffer to hold characters to be displayed on the LCD
 extern char i,x;
 extern char flag;
 extern char slv_addr;
 extern char MST_data;
 extern int  I2C_State;



/******************************************************************************
*	funzione		Int_to_str(unsigned int num)
*
*
*	cosa fa 		converte un valore int (Max 65535) nella corrispondente
*					stringa da inviare a LCD. Poi analizza il risultato e cambia
*					gli 0 antecedenti la prima cifra significativa in ' '
*
******************************************************************************/
void int_to_str(unsigned int num){
	char i;
	unsigned int a, b, n;

	n=10000;
	b =num;
	for(i=0; i<5; i++){
		a = b/n;
		disp[i]=a + 0x30;
		b = b - (a*n);
		n = n/10;
	}
	for(i=0; i<4; i++){		//se  0 anche la 5 cifra lo 0 va a LCD
		if(disp[i] == '0') disp[i] = ' ';
		else break;
	}
	disp[5]=0;				// marca il fine stringa
}

/******************************************************************************
*	funzione		clr_disp_buffer(void)
*
*
*	cosa fa 		clear il buffer per una riga dell'LCD
*
******************************************************************************/

void clr_line_buffer(void){
  for(x = 0; x <16; x++)
    disp[x] = ' ';
}


/******************************************************************************
*	funzione		strobe()
*
*
*	cosa fa 		invia un impulso di strobe al pin 6 (E) dell'LCD tramite
*					Port2.3 La porta  normalmente alta:
******************************************************************************/
void strobe(void){
   P2OUT &= 0xF7; //Enable Low
  __delay_cycles(20);
   P2OUT |= 0x08; //Enable High
}

/******************************************************************************
*	funzione		init_LCD()
*
*
*	cosa fa 		resetta l'LCD e imposta la gestione a 8 linee (38)
*
******************************************************************************/
void init_LCD(void){
   P2OUT &= 0xEF; 				// RS dell'LCD = 0 modo comandi
   write_i2c(0x38);
   __delay_cycles (100); 			// 100 US

   strobe();
    __delay_cycles (100); 			// 100 uS

    strobe();
     __delay_cycles (100); 			// 100 uS

   // Ox01
   write_i2c(0x01);
    __delay_cycles (200); 			 	// 200 uS

    // Ox0E
    write_i2c(0x0E);
     __delay_cycles (200); 			 	// 200 uS


     // Ox0C
     write_i2c(0x0C);
      __delay_cycles (200); 			 	// 200 uS
  }



/******************************************************************************
*	funzione		LCD_wrt()
*
*
*	cosa fa 		manda una variabile di tipo char a LCD.
*					Se tipo = 0  un comando per LCD
*					se tipo  = 1 e un dato da visualizzare
*
******************************************************************************/
void LCD_wrt(char tipo, char dato){
	if(tipo == 0)
	    P2OUT &=  0xEF;
	if(tipo == 1)
		P2OUT |= 0x10;
	write_i2c(dato);
}

/******************************************************************************
*	funzione		LCD_pos_line()
*
*
*	cosa fa 		E' un comando per LCD che definisce su quale riga e in
*					che posizione deve portarsi il cursore
*
******************************************************************************/
void LCD_pos_line(char posizione, char line){
  char dato;
	if(line == 1)
	 dato = 0x80 + posizione-1;
	if(line == 2)
	 dato = 0xC0 + posizione-1;
	if(line == 3)
	 dato = 0x90 + posizione-1;
	if(line == 4)
	 dato = 0xD0 + posizione-1;
	LCD_wrt(0,dato);

	P2OUT &= 0xEF; 			// modo Commandi
    write_i2c(0x0C);
   __delay_cycles (100); 			// 100 uS

}

/******************************************************************************
*	funzione		LCD_string(const char stringa[])
*
*
*	cosa fa 		Manda a LCD una stringa di caratteri costanti
*
*
******************************************************************************/
void LCD_string(const char stringa[]){
	char i=0, dato;

	do{
		dato = stringa[i];
		if(dato == 0) break;
		LCD_wrt(1,dato);
		i++;
	}while(dato != 0);

}

/******************************************************************************
*	funzione		LCD_clear(void)
*
*
*	cosa fa 		Manda a LCD c0me comando 01
*
*
******************************************************************************/
void LCD_clear(void){
  LCD_wrt(0,0x01);
}

/******************************************************************************
*	funzione		write_i2c(char mst_data)
*
*
*	cosa fa 		invia via I2c il dato all'expander
*					poi ilvia uno strobe all'LCD
*
******************************************************************************/
void write_i2c(char mst_data){

	I2C_State = 0;
	MST_data = mst_data;
	  while(flag == 0)
	  {
	    USICTL1 |= USIIFG;          // Set flag and start communication
	    LPM0;                       // CPU off, await USI interrupt
	    _NOP();                     // Used for IAR
	    __delay_cycles (5000);
	  }
  flag = 0;							//ripristino il flag
  strobe();							//impulso di memorizzazione per LCD
}



